// Include all Serd and Sord source files into a single file.
// This is not part of the original libraries.

#include "../serd/src/byte_source.c"
#include "../serd/src/env.c"
#include "../serd/src/node.c"
#include "../serd/src/reader.c"
#include "../serd/src/string.c"
#include "../serd/src/uri.c"
#include "../serd/src/writer.c"

#include "src/sord.c"
#include "src/syntax.c"

